<!--
 ~ SPDX-FileCopyrightText: Copyright DB Netz AG and the capellambse contributors
 ~ SPDX-License-Identifier: Apache-2.0
 -->

{% extends "!layout.html" %}
  {% block head %} {{ super() }}
  <style>
    /* Content-body */
    .wy-body-for-nav {
      color: #204051;
      background: #84a9ac;
    }
    /* Sidebar (and topbar for mobile) */
    .wy-side-nav-search, .wy-menu, .wy-nav-top {
      color: #e4e3e3;
      background: #204051;
    }
    .wy-nav-side {
      background: #204051;
    }
    .version {
      color: #e4e3e3;
    }
    hr {
      border-top: 1px solid #204051;
    }
  </style>
{% endblock %}
